# coding: utf-8

"""
  Video API

  Official API
"""

from __future__ import absolute_import

from videoguys.clients.api import ApiClient
from videoguys.clients.upload import UploadClient
from videoguys.clients.download import DownloadClient

from simple_rest_client.exceptions import ErrorWithResponse